{******************************************************************************}
{*    "DirectX.    Delphi".                  *}
{*  BHV. www.bhv.ru                                               *}
{*  .. softgl@chat.ru                                                *}
{******************************************************************************}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  AppEvnts,
  DirectXGraphics;

type
  TfrmD3D = class(TForm)
    ApplicationEvents1: TApplicationEvents;
    procedure FormCreate(Sender: TObject);
    procedure ApplicationEvents1Minimize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ApplicationEvents1Restore(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FD3D            : IDIRECT3D8;
    FD3DDevice      : IDIRECT3DDEVICE8;
    FD3DVB          : IDIRECT3DVERTEXBUFFER8;
    FActive         : BOOL;
    ThisTickCount   : DWORD;
    LastTickCount   : DWORD;
    function Render : HRESULT;
    function InitD3D : HRESULT;
    function InitPoints : HRESULT;
    function MovePoints : HRESULT;
    procedure Cleanup;
    procedure ErrorOut (const Caption : PChar; const hError : HRESULT);
  public
    { Public declarations }
  end;

type
  TCUSTOMVERTEX = packed record
    X, Y, Z, RHW : Single;
    Color : DWORD;
  end;

const
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZRHW or D3DFVF_DIFFUSE;
  Radius = 100;

var
  frmD3D: TfrmD3D;

  VPoints : Array [0..43] of TCUSTOMVERTEX;

var
  X1, Y1, X2, Y2 : Integer;
  X3, Y3, X4, Y4 : Integer;
  StepX1, StepX3 : Integer;
  StepY2, StepY4 : Integer;

implementation

{$R *.DFM}

procedure TfrmD3D.Cleanup;
begin
  if Assigned (FD3DVB) then begin
     FD3DVB._Release;
     FD3DVB := nil;
  end;

  if Assigned (FD3DDevice) then begin
     FD3DDevice._Release;
     FD3DDevice := nil;
  end;

  if Assigned (FD3D) then begin
     FD3D._Release;
     FD3D := nil;
  end;
end;

procedure TfrmD3D.ErrorOut (const Caption : PChar; const hError : HRESULT);
begin
  FActive := False;
  Cleanup;
  MessageBox (Handle, PChar(DXGErrorString (hError)), Caption, 0);
end;

function TfrmD3D.InitD3D : HRESULT;
var
  d3ddm : TD3DDISPLAYMODE;
  d3dpp : TD3DPRESENT_PARAMETERS;
  hRet : HRESULT;
begin
  FD3DDevice := nil;
  FD3DVB := nil;
  FD3D := nil;

  FD3D := Direct3DCreate8(D3D_SDK_VERSION);
  if FD3D = nil then begin
     Result := E_FAIL;
     Exit;
  end;

  hRet := FD3D.GetAdapterDisplayMode(D3DADAPTER_DEFAULT, d3ddm);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  ZeroMemory(@d3dpp, SizeOf(d3dpp));
  with d3dpp do begin
       Windowed := True;
       SwapEffect := D3DSWAPEFFECT_DISCARD;
       BackBufferFormat := d3ddm.Format;
  end;

  Result := FD3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, Handle,
                              D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                              d3dpp, FD3DDevice);
end;

function TfrmD3D.Render : HRESULT;
var
  hRet : HRESULT;
begin
  if FD3DDevice = nil then begin
     Result := E_FAIL;
     Exit;
  end;

  hRet := FD3DDevice.Clear(0, nil, D3DCLEAR_TARGET, $00FFFFFF, 0.0, 0);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.BeginScene;
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.DrawPrimitive(D3DPT_LINELIST, 0, 22);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.EndScene;
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  Result := FD3DDevice.Present(nil, nil, 0, nil);
end;

procedure TfrmD3D.FormCreate(Sender: TObject);
var
  hRet : HRESULT;
begin
  Randomize;
  hRet := InitD3D;
  if Failed (hRet) then ErrorOut ('InitD3D', hRet);
  hRet := InitPoints;
  if Failed (hRet) then ErrorOut ('InitPoint', hRet);
end;

procedure TfrmD3D.ApplicationEvents1Minimize(Sender: TObject);
begin
  FActive := False;
end;

procedure TfrmD3D.FormActivate(Sender: TObject);
begin
  LastTickCount := GetTickCount;
  FActive := True;
end;

procedure TfrmD3D.ApplicationEvents1Idle(Sender: TObject;
  var Done: Boolean);
var
  hRet : HRESULT;
begin
  if FActive then begin
     hRet := Render;
     if FAILED(hRet) then begin
        FActive := False;
        ErrorOut ('Render', hRet);
        Exit;
     end;

     ThisTickCount := GetTickCount;
     if ThisTickCount - LastTickCount > 30 then begin
        MovePoints;

        LastTickCount := GetTickCount;
     end;
  end;

  Done := False;
end;

procedure TfrmD3D.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then Close else
  if Key = VK_SPACE then begin
     X1 := random (300);
     Y1 := random (300);
     X2 := random (300);
     Y2 := random (300);

     X3 := random (300);
     Y3 := random (300);
     X4 := random (300);
     Y4 := random (300);
  end;
end;

procedure TfrmD3D.ApplicationEvents1Restore(Sender: TObject);
begin
  FActive := True;
end;

procedure TfrmD3D.FormDestroy(Sender: TObject);
begin
  Cleanup
end;

function TfrmD3D.InitPoints : HRESULT;
var
  pVertices : PByte;
  hRet : HRESULT;
  i : Integer;
begin
  X1 := 0;
  Y1 := 0;
  X2 := 0;
  Y2 := 300;

  X3 := 0;
  Y3 := 300;
  X4 := 300;
  Y4 := 300;

  StepX1 := random (4) - 4;
  StepX3 := random (4) - 4;

  StepY2 := random (4) - 4;
  StepY4 := random (4) - 4;

  for i := 0 to 10 do begin
      with VPoints [i * 2] do begin
           X := X1 + i * (X2 - X1) / 10;
           Y := Y1 + i * (Y2 - Y1) / 10;
           Z := 0.0;
           RHW := 0.0;
           Color := D3DCOLOR_XRGB(255, 0, 0);
      end;
      with VPoints [i * 2 + 1] do begin
           X := X3 + i * (X4 - X3) / 10;
           Y := Y3 + i * (Y4 - Y3) / 10;
           Z := 0.0;
           RHW := 0.0;
           Color := D3DCOLOR_XRGB(255, 0, 0);
      end;
  end;

  for i := 0 to 10 do begin
      with VPoints [i * 2 + 22] do begin
           X := X1 + i * (X3 - X1) / 10;
           Y := Y1 + i * (Y3 - Y1) / 10;
           Z := 0.0;
           RHW := 0.0;
           Color := D3DCOLOR_XRGB(255, 0, 0);
      end;
      with VPoints [i * 2 + 1 + 22] do begin
           X := X2 + i * (X4 - X2) / 10;
           Y := Y2 + i * (Y4 - Y2) / 10;
           Z := 0.0;
           RHW := 0.0;
           Color := D3DCOLOR_XRGB(255, 0, 0);
      end;
  end;

  hRet := FD3DDevice.CreateVertexBuffer(SizeOf(VPoints),
                                        D3DUSAGE_WRITEONLY, D3DFVF_CUSTOMVERTEX,
                                        D3DPOOL_DEFAULT, FD3DVB);
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DVB.Lock(0, SizeOf(VPoints), pVertices, 0);
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  Move (VPoints, pVertices^, SizeOf(VPoints));

  hRet := FD3DVB.Unlock;
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.SetStreamSource(0, FD3DVB, SizeOf(TCUSTOMVERTEX));
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  Result := FD3DDevice.SetVertexShader(D3DFVF_CUSTOMVERTEX);
end;

function TfrmD3D.MovePoints : HRESULT;
var
  pVertices : PByte;
  hRet : HRESULT;
  i : Integer;
begin
  X1 := X1 + StepX1;
  if X1 < 0 then begin
     X1 := 0;
     StepX1 := -StepX1;
  end;
  if X1 > 300 then begin
     X1 := 300;
     StepX1 := -StepX1;
  end;
  Y2 := Y2 + StepY2;
  if Y2 < 0 then begin
     Y2 := 0;
     StepY2 := -StepY2;
  end;
  if Y2 > 300 then begin
     Y2 := 300;
     StepY2 := -StepY2;
  end;
  X3 := X3 + StepX3;
  if X3 < 0 then begin
     X3 := 0;
     StepX3 := -StepX3;
  end;
  if X3 > 300 then begin
     X3 := 300;
     StepX3 := -StepX3;
  end;
  Y4 := Y4 + StepY4;
  if Y4 < 0 then begin
     Y4 := 0;
     StepY4 := -StepY4;
  end;
  if Y4 > 300 then begin
     Y4 := 300;
     StepY4 := -StepY4;
  end;
  for i := 0 to 10 do begin
      with VPoints [i * 2] do begin
           X := X1 + i * (X2 - X1) / 10;
           Y := Y1 + i * (Y2 - Y1) / 10;
      end;
      with VPoints [i * 2 + 1] do begin
           X := X3 + i * (X4 - X3) / 10;
           Y := Y3 + i * (Y4 - Y3) / 10;
      end;
  end;

  for i := 0 to 10 do begin
      with VPoints [i * 2 + 22] do begin
           X := X1 + i * (X3 - X1) / 10;
           Y := Y1 + i * (Y3 - Y1) / 10;
      end;
      with VPoints [i * 2 + 1 + 22] do begin
           X := X2 + i * (X4 - X2) / 10;
           Y := Y2 + i * (Y4 - Y2) / 10;
      end;
  end;

  hRet := FD3DVB.Lock(0, SizeOf(VPoints), pVertices, 0);
  if Failed (hRet) then begin
     Result := hRet;
     Exit;
  end;

  Move (VPoints, pVertices^, SizeOf(VPoints));

  Result := FD3DVB.Unlock;
end;

end.
